using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Diagnostics;
using System.Xml;

namespace ExpressionTree
{
	public class FormMain : System.Windows.Forms.Form
	{

		TNode term;
		string fileName = "Terms.xml";

		private System.Windows.Forms.Button btnCreate;
		private System.Windows.Forms.Button btnWriteAsXml;
		private System.Windows.Forms.TreeView treeView;
		private System.Windows.Forms.Button btnRead;
		private System.ComponentModel.Container components = null;

		public FormMain()
		{
			InitializeComponent();
		}

		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.btnCreate = new System.Windows.Forms.Button();
			this.btnRead = new System.Windows.Forms.Button();
			this.btnWriteAsXml = new System.Windows.Forms.Button();
			this.treeView = new System.Windows.Forms.TreeView();
			this.SuspendLayout();
			// 
			// btnCreate
			// 
			this.btnCreate.Location = new System.Drawing.Point(8, 240);
			this.btnCreate.Name = "btnCreate";
			this.btnCreate.Size = new System.Drawing.Size(72, 24);
			this.btnCreate.TabIndex = 0;
			this.btnCreate.Text = "Create";
			this.btnCreate.Click += new System.EventHandler(this.btnCreate_Click);
			// 
			// btnRead
			// 
			this.btnRead.Location = new System.Drawing.Point(192, 240);
			this.btnRead.Name = "btnRead";
			this.btnRead.Size = new System.Drawing.Size(72, 24);
			this.btnRead.TabIndex = 0;
			this.btnRead.Text = "Read";
			this.btnRead.Click += new System.EventHandler(this.btnRead_Click);
			// 
			// btnWriteAsXml
			// 
			this.btnWriteAsXml.Location = new System.Drawing.Point(96, 240);
			this.btnWriteAsXml.Name = "btnWriteAsXml";
			this.btnWriteAsXml.Size = new System.Drawing.Size(80, 24);
			this.btnWriteAsXml.TabIndex = 0;
			this.btnWriteAsXml.Text = "Save as XML";
			this.btnWriteAsXml.Click += new System.EventHandler(this.btnWriteAsXml_Click);
			// 
			// treeView
			// 
			this.treeView.ImageIndex = -1;
			this.treeView.Location = new System.Drawing.Point(8, 8);
			this.treeView.Name = "treeView";
			this.treeView.SelectedImageIndex = -1;
			this.treeView.Size = new System.Drawing.Size(256, 224);
			this.treeView.TabIndex = 1;
			// 
			// FormMain
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(432, 273);
			this.Controls.Add(this.treeView);
			this.Controls.Add(this.btnCreate);
			this.Controls.Add(this.btnRead);
			this.Controls.Add(this.btnWriteAsXml);
			this.Name = "FormMain";
			this.Text = "Form1";
			this.ResumeLayout(false);

		}
		#endregion

		[STAThread]
		static void Main() 
		{
			Application.Run(new FormMain());
		}

		void CreateXMLFile()
		{
			XmlTextWriter xw;
			// Create a new XmlTextWriter.
			xw = new XmlTextWriter(fileName, System.Text.Encoding.UTF8);
			xw.Formatting = Formatting.Indented;
			xw.Indentation = 1;
			xw.IndentChar = '\t';

			xw.WriteStartDocument();
			xw.WriteStartElement("Terms");

			term.WriteAsXMLNode(xw);
			
			xw.WriteEndElement();
			xw.WriteEndDocument();
			xw.Close();
		}

		void ReadFromXMLFile()
		{
			XmlDocument xmlDoc = new XmlDocument();
			try 
			{
				xmlDoc.Load(fileName);
			}
			catch (Exception e)
			{
				Debug.WriteLine(e.Message);
				return;
			}
			XmlNode node = xmlDoc.SelectSingleNode("/Terms/Term");
			if (node == null)
				return;
			term = new TNode();
			AddTermFromXmlDoc(term, node);
			term.Calculate();
			GenerateTree();
		}

		void AddTermFromXmlDoc(TNode term, XmlNode node)
		{
			XmlNode xNode;
			xNode = node.SelectSingleNode("@OpCode");
			string s;
			s = xNode != null ? xNode.InnerText : TNode.terminateOpCode.ToString();
			term.OpCode = s[0];
			xNode = node.SelectSingleNode("Value");
			s = xNode != null ? xNode.InnerText : "NaN";
			term.Result = XmlConvert.ToDouble(s);
			XmlNodeList nodeList = node.SelectNodes("Term");
			if (nodeList == null || nodeList.Count < 2)
				return;
			TNode termA = new TNode();
			term.OpA = termA;
			AddTermFromXmlDoc(termA, nodeList[0]);
			TNode termB = new TNode();
			term.OpB = termB;
			AddTermFromXmlDoc(termB, nodeList[1]);
		}

		void GenerateTree()
		{
			treeView.Nodes.Clear();
			TreeNode tn = new TreeNode(term.ToString());
			tn.Tag = term;
			treeView.Nodes.Add(tn);
			AddChildsToTree(tn, term);
			tn.ExpandAll();
		}

		void AddChildsToTree(TreeNode tn, TNode term)
		{
			TreeNode tChild;
			if (term.OpA != null)
			{
				tChild = new TreeNode(term.OpA.ToString());
				tChild.Tag = term.OpA;
				tn.Nodes.Add(tChild);
				AddChildsToTree(tChild, term.OpA);
			}
			if (term.OpB != null)
			{
				tChild = new TreeNode(term.OpB.ToString());
				tn.Nodes.Add(tChild);
				tChild.Tag = term.OpB;
				AddChildsToTree(tChild, term.OpB);
			}
		}

		private void btnCreate_Click(object sender, System.EventArgs e)
		{
			term = new TNode();
			term.AddExpression('*');
			double r;
			TNode subTerm;
			subTerm = term.OpA;
			subTerm.AddExpression(3, 4, '+');
			subTerm = term.OpB;
			subTerm.AddExpression(5, 2, '-');
			r = term.Calculate();
			Debug.WriteLine(String.Format("r = {0:f2}", r));
			GenerateTree();
		}

		private void btnWriteAsXml_Click(object sender, System.EventArgs e)
		{
			CreateXMLFile();
		}

		private void btnRead_Click(object sender, System.EventArgs e)
		{
			ReadFromXMLFile();
		}
	}
}
